@@ -21,9 +21,12 @@ from typing import Iterable, Optional, Text
 import uuid
 
 from cascades._src.distributions import base as dists
+import jax
 import openai
 
-openai.api_key = os.getenv('OPENAI_API_KEY')
+api_key = os.getenv('OPENAI_API_KEY')
+if api_key:
+  openai.api_key = api_key
 
 
 # TODO(ddohan): Persist cache to disk
@@ -41,7 +44,7 @@ class GPT(dists.Distribution):
 
   stop: Optional[Iterable[Text]] = ('\n',)
 
-  engine = 'davinci-codex'
+  engine: str = 'davinci-codex'
   temperature: float = 0.7
   max_tokens: int = 128
   top_p: float = .95
@@ -60,8 +63,11 @@ class GPT(dists.Distribution):
     """
     if rng is None:
       rng = uuid.uuid4().hex
+    elif isinstance(rng, jax.numpy.DeviceArray):
+      rng = jax.random.randint(rng, (), 0, 1_000_000_000)
+      rng = int(rng)
     elif not isinstance(rng, int):
-      raise ValueError('RNG must be an integer or Jax key.')
+      raise ValueError(f'RNG must be an integer or Jax key. Was {rng}')
     result = cached_completion(
         rng=rng,
         model=self.engine,
