@@ -70,7 +70,7 @@ def forward_sample(fn, seed, *args, **kwargs):
   # gen_fn = ApplyContext(lambda: fn(*args, **kwargs))
   gen_fn = lambda: fn(*args, **kwargs)
   # TODO(ddohan): Add in ParamHandler
-  handler_fn = compose_handlers([Record, StopOnReject, Observer, Sampler])
+  handler_fn = compose_handlers([Record, StopOnReject, Sampler, Observer])
   forward_sample_handler: Record = handler_fn(Seed(seed=seed, gen_fn=gen_fn))  # pytype: disable=annotation-type-mismatch
   result_with_metadata = forward_sample_handler.run_with_intermediates(
       verbose=False)
@@ -143,10 +143,7 @@ def reject(reason, name=None):
   """Add -inf term to likelihood of current trace."""
   dist = dists.Factor(reason=reason, factor=-math.inf)
   yield Reject(
-      fn=dist,
-      value=dists.FactorSentinel,
-      score=-math.inf,
-      name=name or 'reject')
+      fn=dist, value=dists.FactorSentinel, score=-math.inf, name=name or reason)
 
 
 def param(name=None, dist=None, value=None):
@@ -159,9 +156,14 @@ def _yielded_value_to_effect(value):
   """Convert a value to an effect."""
   if isinstance(value, Effect):
     return value
-  elif isinstance(value, (Distribution, dists.np_dists.Distribution)):
+  elif isinstance(value, dists.np_dists.Distribution):
+    name = str(value.__class__)
+    if hasattr(value, 'name'):
+      name = value.name
     # pylint does not handle dataclass inheritance properly.
-    return Sample(fn=value)  # pylint: disable=unexpected-keyword-arg
+    return Sample(fn=value, name=name)  # pylint: disable=unexpected-keyword-arg
+  elif isinstance(value, Distribution):
+    return Sample(fn=value, name=value.name)  # pylint: disable=unexpected-keyword-arg
   else:
     raise ValueError('Unknown effect type %s' % str(value))
 
@@ -176,15 +178,16 @@ class Effect:
   name: Optional[Text] = None
 
   # Results of sampling or scoring.
-  value: Optional[Any] = dataclasses.field(repr=True, default=None)
   score: Optional[float] = dataclasses.field(repr=True, default=None)
+  value: Optional[Any] = dataclasses.field(repr=True, default=None)
 
   # Callable, generally used for sampling and scoring.
   fn: Optional[Union[Callable, Distribution]] = dataclasses.field(  # pylint: disable=g-bare-generic
-      repr=False,
-      default=None)
-  args: Optional[List[Any]] = dataclasses.field(default_factory=list)
-  kwargs: Optional[Dict[Text, Any]] = dataclasses.field(default_factory=dict)
+      repr=False, default=None)
+  args: Optional[List[Any]] = dataclasses.field(
+      repr=False, default_factory=list)
+  kwargs: Optional[Dict[Text, Any]] = dataclasses.field(
+      repr=False, default_factory=dict)
 
   # If True, then the generator should be halted
   # generally used when likelihood becomes infinite.
@@ -332,7 +335,9 @@ class EffectHandler:
           effect = self.process(effect)
           if effect is None:
             raise ValueError(f'Did not return effect from {self}')
-          yield effect
+          returned_effect = yield effect
+          if returned_effect is not None:
+            effect = returned_effect
 
           # TODO(ddohan): postprocess will be applied from outside in
           # Do we want to somehow apply it inside out as well?
@@ -394,6 +399,9 @@ class Record(EffectHandler):
 
   def __repr__(self):
     kvs = [f'  {k}: {v}' for k, v in self.trace.items()]
+    if self.done:
+      kvs.append(f'  return: {self.return_value}')
+
     kvs = '\n'.join(kvs)
     return f'Record(\n{kvs}\n)'
 
@@ -457,12 +465,31 @@ class Sampler(EffectHandler):
 class Observer(EffectHandler):
   """Compute score of Observe statements."""
 
-  def __init__(self, gen_fn=None, await_timeout=None, rescore=True):
+  def __init__(self,
+               gen_fn=None,
+               await_timeout=None,
+               rescore=True,
+               observed=None):
     super().__init__(gen_fn=gen_fn)
     self._await_timeout = await_timeout
     self._rescore = rescore
+    self._observed = observed
 
   def process(self, effect):
+    if self._observed and effect.name in self._observed:
+      expected = self._observed[effect.name]
+      if isinstance(effect, Log):
+        if effect.value != expected:
+          effect.metadata = dict(expected=expected)
+          effect.score = -math.inf
+      elif isinstance(effect, Observe):
+        effect.value = expected
+        effect.score = None
+      elif isinstance(effect, Sample):
+        effect = Observe(**effect.__dict__)
+        effect.value = expected
+        effect.score = None
+
     if isinstance(effect, Observe):
       if effect.value is None:
         raise ValueError(f'Observe with a None value: {effect}')
@@ -487,7 +514,7 @@ class StopOnReject(EffectHandler):
   """Mark that tracing should stop in event of inf likelihood."""
 
   def process(self, effect):
-    if (isinstance(effect, Observe) and effect.score is not None):
+    if effect.score is not None:
       should_stop = jax.numpy.any(jax.numpy.isinf(effect.score))
       if jax.device_get(should_stop):
         effect.should_stop = True
